//
//  EventEmitter.swift
//
//  Copyright © 2022 Anagog Ltd. All rights reserved.
//

import Foundation
import Flutter
import JedAIKit

class EventEmitter: NSObject, FlutterStreamHandler {
    
    static let shared = EventEmitter()
    
    private static let eventsChannelName = "com.anagog.flutter/method_channel/events"
    
    private var channelEvents: FlutterEventChannel? = nil
    private var eventSink: FlutterEventSink? = nil
    
    static func register(with registrar: FlutterPluginRegistrar) {
        os_log_file("[pacakge] EventEmitter register")
        guard shared.channelEvents == nil else {
            return
        }
        shared.channelEvents = FlutterEventChannel(name: eventsChannelName, binaryMessenger: registrar.messenger())
        shared.channelEvents?.setStreamHandler(shared)
        
        EventEmitterManager.shared.delegate = shared
        EventEmitterManager.shared.checkForSend()
    }
    
    static func unregister() {
        os_log_file("[pacakge] EventEmitter unregister")
        shared.channelEvents = nil
        shared.eventSink = nil
    }
    
    func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        os_log_file("[pacakge] EventEmitter start listen")
        self.eventSink = events
        EventEmitterManager.shared.checkForSend()
        return nil
    }
    
    func onCancel(withArguments arguments: Any?) -> FlutterError? {
        os_log_file("[pacakge] EventEmitter stop listen")
        self.eventSink = nil
        return nil
    }
    
}

extension EventEmitter: EventEmitterManagerDelegate {
    var hasListeners: Bool { eventSink != nil }
    
    func sendEvent(action: EventEmitterAction, payload: Any?) {
        os_log_file("[package] EventEmitter send event %@", action.rawValue)
        self.eventSink?([
            "action": action.rawValue,
            "payload": payload ?? [:]
        ])
    }
}
