//
//  JedAIBridge.swift
//
//  Copyright © 2022 Anagog Ltd. All rights reserved.
//

import Foundation

class JedAIBridge {
    
    static func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
            // MARK: Campaigns
        case "syncCampaigns":
            do {
                try AnagogBridgeAPI.shared.downloadCampaigns()
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
        case "simulateCampaign":
            do {
                try AnagogBridgeAPI.shared.simulateCampaign((call.arguments as? [String: String])?["campaign_identifier"])
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
        case "getCampaigns":
            AnagogBridgeAPI.shared.getCampaigns({ res in
                switch res {
                case .success(let campaigns):
                    result(campaigns)
                case .failure(let error):
                    result(FlutterError.error(error))
                }
            })
            
            
            // MARK: Micromoments
        case "fireMicromoment":
            guard let micromoment = call.arguments as? [String: Any] else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            do {
                try AnagogBridgeAPI.shared.fireMicromoment(micromoment)
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
            
            // MARK: Microsegments
        case "getUserDefinedString":
            guard let key = call.arguments as? String else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            AnagogBridgeAPI.shared.getUserDefinedString(key) { res in
                switch res {
                case .success(let value):
                    result(value)
                case .failure(let error):
                    result(FlutterError.error(error))
                }
            }
            
        case "getUserDefinedDecimal":
            guard let key = call.arguments as? String else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            AnagogBridgeAPI.shared.getUserDefinedDecimal(key) { res in
                switch res {
                case .success(let value):
                    result(value)
                case .failure(let error):
                    result(FlutterError.error(error))
                }
            }
            
        case "getUserDefinedInteger":
            guard let key = call.arguments as? String else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            AnagogBridgeAPI.shared.getUserDefinedInteger(key) { res in
                switch res {
                case .success(let value):
                    result(value)
                case .failure(let error):
                    result(FlutterError.error(error))
                }
            }
            
        case "setUserDefinedString":
            guard let arguments = call.arguments as? [String: Any],
                  let key = arguments["key"] as? String,
                  let value = arguments["value"] as? String
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            AnagogBridgeAPI.shared.setUserDefinedString(key, value: value, callback: result)
            
        case "setUserDefinedDecimal":
            guard let arguments = call.arguments as? [String: Any],
                  let key = arguments["key"] as? String,
                  let value = arguments["value"] as? Double
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            AnagogBridgeAPI.shared.setUserDefinedDecimal(key, value: value, callback: result)
            
        case "setUserDefinedInteger":
            guard let arguments = call.arguments as? [String: Any],
                  let key = arguments["key"] as? String,
                  let value = arguments["value"] as? Int
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            AnagogBridgeAPI.shared.setUserDefinedInteger(key, value: value, callback: result)
            
        case "getUserDefinedStat": // deprecated
            result([String: Any]())
            
            
            // MARK: Reports
        case "forceScheduleReports":
            do {
                try AnagogBridgeAPI.shared.forceScheduleReports()
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
        case "requestMicrosegmentsSnapshotReport":
            do {
                try AnagogBridgeAPI.shared.requestMicrosegmentsSnapshotReport()
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
            
            // MARK: JedAI
        case "enableJedAI":
            AnagogBridgeAPI.shared.enableJedAI {
                result(nil)
            }
            
        case "disableJedAI":
            AnagogBridgeAPI.shared.disableJedAI {
                result(nil)
            }
            
        case "disableAndPurgeJedAI":
            AnagogBridgeAPI.shared.disableAndPurgeJedAI {
                result(nil)
            }
            
        case "getJedAIStatus":
            result(AnagogBridgeAPI.shared.getJedAIStatus())
            
        case "share":
            do {
                try AnagogBridgeAPI.shared.share {
                    result(nil)
                }
            } catch {
                result(FlutterError.error(error))
            }
            
        case "getVersion":
            result(AnagogBridgeAPI.shared.getVersion())
            
        case "openDebuggingScreen":
            AnagogBridgeAPI.shared.openDebuggingScreen(completion: result)
            
            
            // MARK: Onboarding
        case "onboardingStart":
            do {
                try AnagogBridgeAPI.shared.onboardingStart()
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
        case "onboardingComplete":
            do {
                try AnagogBridgeAPI.shared.onboardingComplete()
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
        case "onboardingMarkAsCompleted":
            do {
                try AnagogBridgeAPI.shared.onboardingMarkAsCompleted()
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
        case "onboardingReset":
            do {
                try AnagogBridgeAPI.shared.onboardingReset()
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
            
            // MARK: PageTracker
        case "enterPage":
            guard let arguments = call.arguments as? [String: Any],
                  let pageName = arguments["pageName"] as? String
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            do {
                try AnagogBridgeAPI.shared.enterPage(pageName)
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
        case "exitPage":
            do {
                try AnagogBridgeAPI.shared.exitPage()
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
            
            // MARK: Other
        case "sendUserInteractionEvent":
            guard let arguments = call.arguments as? [String: Any],
                  let campaignIdentifier = arguments["campaign_identifier"] as? String,
                  let triggerType = arguments["trigger_type"] as? String
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            do {
                try AnagogBridgeAPI.shared.sendUserInteractionEvent(campaignIdentifier: campaignIdentifier, triggerType: triggerType)
                result(nil)
            } catch {
                result(FlutterError.error(error))
            }
            
        case "handleUNUserNotificationCenter":
            AnagogBridgeAPI.shared.handleUNUserNotificationCenter()
            result(nil)
            
            
            // MARK: Logs
        case "logInfo":
            guard let arguments = call.arguments as? [String: Any],
                  let message = arguments["message"] as? String
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            AnagogBridgeAPI.shared.logInfo("[flutter] \(message)")
            result(nil)
            
        case "logError":
            guard let arguments = call.arguments as? [String: Any],
                  let message = arguments["message"] as? String
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            AnagogBridgeAPI.shared.logError("[flutter] \(message)")
            result(nil)
            
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
}

fileprivate extension FlutterError {
    static func wrongArguments(_ arguments: Any?) -> FlutterError {
        return FlutterError(code: "", message: "Unable to correctly parse input arguments", details: arguments)
    }
    static func error(_ error: Error) -> FlutterError {
        return FlutterError(code: "", message: "Error", details: error.localizedDescription)
    }
}
