//
//  ShareService.swift
//
//  Copyright © 2022 Anagog Ltd. All rights reserved.
//

import UIKit
import JedAIKit

typealias ShareCompletion = UIActivityViewController.CompletionWithItemsHandler

enum App {
    struct FilePaths {
        static var documents: URL {
            let path = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first!
            return URL(fileURLWithPath: path)
        }
        static var caches: URL {
            let path = NSSearchPathForDirectoriesInDomains(.cachesDirectory, .userDomainMask, true).first!
            return URL(fileURLWithPath: path)
        }
        static var temp: URL {
            return URL(fileURLWithPath: NSTemporaryDirectory())
        }
    }
}

/**
 This class is used by the sample app to export a zip file that contains debug information
 for troubleshooting and analysis.
 */
class ShareService {
    
    struct Constants {
        static let encPhrase = "f33dback"
        static let insightsMessage = "Your insights are attached and are ready to be sent for analysis."
    }
    
    // MARK: - Public
    
    class func shareDatabaseAndLogs(in controller: UIViewController? = UIApplication.shared.keyWindow?.rootViewController, completion: ShareCompletion? = nil) {
        guard let controller = controller else { return }
        
        DispatchQueue.global(qos: .userInitiated).async {
            let zipUrl = App.FilePaths.temp.appendingPathComponent("database+logs.zip")
            do {
                try JedAI.shared.share.share(to: zipUrl, options: [.db, .logs, .jema], customSources: nil, password: ShareService.Constants.encPhrase)
            }
            catch {
                os_log_file("[package] Share process failed. (%@)", type:.error, error.fullLocalizedDescription)
            }
            
            DispatchQueue.main.async {
                shareFiles(from: zipUrl, in: controller, completion: completion)
            }
        }
    }
    
    private class func shareFiles(from url: URL, in controller: UIViewController? = UIApplication.shared.keyWindow?.rootViewController, completion: ShareCompletion? = nil) {
        let text = Constants.insightsMessage
        let subject = "Insights"
        let activityViewController = UIActivityViewController(activityItems: [text, url], applicationActivities: nil)
        activityViewController.setValue(subject, forKey: "Subject")
        activityViewController.completionWithItemsHandler = { (activityType, completed, returnedItems, activityError) in
            completion?(activityType, completed, returnedItems, activityError)
        }
        controller?.present(activityViewController, animated: true, completion: nil)
    }
}
