library anagog_flutter;

import 'package:anagog_flutter/logger/logger.dart';
import 'package:anagog_flutter/onboarding/onboarding.dart';
import 'package:anagog_flutter/page_tracker/page_tracker.dart';
import 'package:anagog_flutter/platform_interface/anagog_flutter_platform_interface.dart';
import 'package:anagog_flutter/platform_interface/model/campaign.dart';
import 'package:anagog_flutter/platform_interface/model/constants.dart';
import 'package:anagog_flutter/platform_interface/model/event_type.dart';
import 'package:anagog_flutter/platform_interface/model/micromoment.dart';
import 'package:flutter/services.dart';

export 'package:anagog_flutter/platform_interface/model/campaign.dart';
export 'package:anagog_flutter/platform_interface/model/micromoment.dart';
export 'package:anagog_flutter/platform_interface/model/location.dart';
export 'package:anagog_flutter/platform_interface/model/constants.dart';
export 'package:anagog_flutter/platform_interface/model/event_type.dart';

class AnagogFlutter extends AnagogFlutterPlatform {
  AnagogFlutter._();
  static AnagogFlutter? _instance;
  static AnagogFlutter get instance => _instance ??= AnagogFlutter._();

  static AnagogFlutterOnboarding? _onboarding;
  static AnagogFlutterOnboarding get onboarding =>
      _onboarding ??= OnBoardingImpl();

  static AnagogFlutterPageTracker? _pageTracker;
  static AnagogFlutterPageTracker get pageTracker =>
      _pageTracker ??= AnagogFlutterPageTrackerImpl();

  static AnagogFlutterLogger? _logger;
  static AnagogFlutterLogger get logger =>
      _logger ??= AnagogFlutterLoggerImpl();

  ///Simulate campaign occuranc, use this API to test your camaigns as defined.
  ///[campaignIdentifier] A campaign identifier to simulate. If null simulates all campaigns
  @override
  Future simulateCampaign(String? campaignIdentifier) async =>
      AnagogFlutterPlatform.instance.simulateCampaign(campaignIdentifier);

  ///Sync campaigns
  ///Download all your campaigns to your app
  @override
  Future syncCampaigns() async =>
      AnagogFlutterPlatform.instance.syncCampaigns();

  ///Create an EventStream from SDK to your Flutter app.
  ///Use this API to listen for SDK events such as onCampaignTriggered, onNotificationClick
  @override
  Stream obtainEventStream() =>
      AnagogFlutterPlatform.instance.obtainEventStream();

  ///Get the app custom micro segments map
  @override
  Future<Map<dynamic, dynamic>?> getUserDefinedStat() async =>
      AnagogFlutterPlatform.instance.getUserDefinedStat();

  ///Set String custom micro segment value
  ///[arguments] Micro Segment values
  @override
  Future<void> setUserDefinedString(String name, String value) async =>
      AnagogFlutterPlatform.instance.setUserDefinedString(name, value);

  ///Get String custom micro segment
  ///[name] Stat name
  @override
  Future<String?> getUserDefinedString(String name) async =>
      AnagogFlutterPlatform.instance.getUserDefinedString(name);

  ///Set Integer custom micro segment value
  ///[arguments] Micro Segment values
  @override
  Future<void> setUserDefinedInteger(String name, int value) async =>
      AnagogFlutterPlatform.instance.setUserDefinedInteger(name, value);

  ///Get Integer custom micro segment
  ///[name] Stat name
  @override
  Future<int?> getUserDefinedInteger(String name) async =>
      AnagogFlutterPlatform.instance.getUserDefinedInteger(name);

  ///Set Decimal custom micro segment
  ///[arguments] Micro Segment values
  @override
  Future<void> setUserDefinedDecimal(String name, double value) async =>
      AnagogFlutterPlatform.instance.setUserDefinedDecimal(name, value);

  ///Get Decimal custom micro segment
  ///[name] Stat name
  @override
  Future<double?> getUserDefinedDecimal(String name) async =>
      AnagogFlutterPlatform.instance.getUserDefinedDecimal(name);

  ///Use to create report a [Micromoment] object from your app
  @override
  Future fireMicromoments(Micromoment micromoment) async =>
      AnagogFlutterPlatform.instance.fireMicromoments(micromoment);

  ///Create and share SDK report.
  ///Use this in case you want to share with Anagog SDK operation and debugging report for analyze
  @override
  Future share() async => AnagogFlutterPlatform.instance.share();

  ///Get SDK Version
  @override
  Future getJedAIVersion() async =>
      AnagogFlutterPlatform.instance.getJedAIVersion();

  ///Request to generate Microsegments Snapshot report.
  @override
  Future requestMicrosegmentsSnapshotReport() async =>
      AnagogFlutterPlatform.instance.requestMicrosegmentsSnapshotReport();

  ///Generate and send SDK reports
  @override
  Future forceScheduleReports() async =>
      AnagogFlutterPlatform.instance.forceScheduleReports();

  ///Forward user interaction event to SDK
  @override
  Future setUserInteractionEvent(
          String campaignIdentifier, AnagogActivityType activityType) async =>
      AnagogFlutterPlatform.instance
          .setUserInteractionEvent(campaignIdentifier, activityType);

  ///Get available campaigns list
  @override
  Future<List<Campaign>?> getCampaigns() async =>
      AnagogFlutterPlatform.instance.getCampaigns();

  ///Get the plugin version
  @override
  Future getPluginVersion() async {
    final fileContent = await rootBundle.loadString(
      "packages/anagog_flutter/pubspec.yaml",
    );

    if (fileContent.isNotEmpty) {
      String versionStr = fileContent.substring(
          fileContent.indexOf("version:"), fileContent.length);
      versionStr = versionStr.substring(0, versionStr.indexOf("\n"));
      return versionStr
          .substring(versionStr.indexOf(":") + 1, versionStr.length)
          .trim();
    }
    return null;
  }

  ///Returns JedAI SDK state
  @override
  Future<JedAIStatus> getJedAIStatus() async =>
      AnagogFlutterPlatform.instance.getJedAIStatus();

  ///Enable and start JedAI SDK
  @override
  Future enableJedAI() async => AnagogFlutterPlatform.instance.enableJedAI();

  ///Disable and stop JedAI SDK
  @override
  Future disableJedAI() async => AnagogFlutterPlatform.instance.disableJedAI();

  ///Disable and stop JedAI. Also, all stored data by JedAI will be removed
  @override
  Future disableAndPurgeJedAI() async =>
      AnagogFlutterPlatform.instance.disableAndPurgeJedAI();

  ///Opens JedAI SDK debugging screen
  @override
  Future openDebuggingScreen() async =>
      AnagogFlutterPlatform.instance.openDebuggingScreen();
}
