import 'package:anagog_flutter/platform_interface/model/location.dart';

///Anagog SDK MicroMoment object, represents campaign Micromoment event which app can pass to SDK
///[identifier] Micromoment identifier such as: 'Registration'
///[textParameters] A Map of Micromoment textParameters such as: {'Status': 'Completed'}
///[numericParameters] A Map of Micromoment numeric values such as: {'Score': 150}
///[locationParameters] A Map of Micromoment [Location] values such as: {'Home': Location(latitude: 32.5, longitude: 34.6)}
class Micromoment {
  final String identifier;
  final Map<String, String> textParameters;
  final Map<String, double> numericParameters;
  final Map<String, Location> locationParameters;
  final Map<String, bool> booleanParameters;

  Micromoment(
      {required this.identifier,
      this.textParameters = const {},
      this.numericParameters = const {},
      this.locationParameters = const {},
      required this.booleanParameters});

  Map<String, dynamic> toJson() => {
        'identifier': identifier,
        'textParameters': textParameters,
        'numericParameters': numericParameters,
        'booleanParameters': booleanParameters,
        'locationParameters':
            locationParameters.map((k, e) => MapEntry(k, e.toJson()))
      };

  factory Micromoment.fromJson(Map<String, dynamic> json) {
    return Micromoment(
        identifier: json['identifier'] as String,
        textParameters: Map<String, String>.from(json['textParameters'] as Map),
        numericParameters:
            (json['numericParameters'] as Map<String, dynamic>).map(
          (k, e) => MapEntry(k, (e as num).toDouble()),
        ),
        locationParameters:
            (json['locationParameters'] as Map<String, dynamic>).map(
          (k, e) => MapEntry(k, Location.fromJson(e as Map<String, dynamic>)),
        ),
        booleanParameters:
            Map<String, bool>.from(json['booleanParameters'] as Map));
  }

  @override
  bool operator ==(Object other) =>
      other is Micromoment &&
      other.runtimeType == runtimeType &&
      other.identifier == identifier &&
      other.textParameters == textParameters &&
      other.numericParameters == numericParameters &&
      other.locationParameters == locationParameters &&
      other.booleanParameters == booleanParameters;

  @override
  int get hashCode =>
      31 *
      (identifier.hashCode +
          numericParameters.hashCode +
          textParameters.hashCode +
          locationParameters.hashCode +
          booleanParameters.hashCode);

  @override
  String toString() {
    return """{
       identifier: $identifier\n
       textParameters: $textParameters\n
       numericParameters: $numericParameters\n
       booleanParameters: $booleanParameters\n
       locationParameters: $locationParameters
    }""";
  }

  Micromoment copyWith(
          {String? identifier,
          Map<String, String>? textParameters,
          Map<String, double>? numericParameters,
          Map<String, Location>? locationParameters,
          Map<String, bool>? booleanParameters}) =>
      Micromoment(
          identifier: identifier ?? this.identifier,
          textParameters: textParameters ?? this.textParameters,
          numericParameters: numericParameters ?? this.numericParameters,
          locationParameters: locationParameters ?? this.locationParameters,
          booleanParameters: booleanParameters ?? this.booleanParameters);
}
